// Programming 2D Games
// Copyright (c) 2011 by: 
// Charles Kelly
// createThisClass.cpp v1.1
// This class is the core of the game

#include "pong.h"
#include "graphics.h"


//=============================================================================
// Constructor
//=============================================================================
Pong::Pong()
{
    dxFont = new TextDX();  // DirectX font
	score1 = 0;
	score2 = 0;
	winner = -1;


}

//=============================================================================
// Destructor
//=============================================================================
Pong::~Pong()
{
    releaseAll();           // call onLostDevice() for every graphics item
    safeDelete(dxFont);
}

//=============================================================================
// Initializes the game
// Throws GameError on error
//=============================================================================
void Pong::initialize(HWND hwnd)
{
    Game::initialize(hwnd); // throws GameError

	// font texture
	 if(dxFont->initialize(graphics, 16, true, false, "Arial") == false)
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing DirectX font"));

	// ball texture
	if (!ballTexture.initialize(graphics,BALL_IMAGE))
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing paddle texture")); 

	// paddle1 texture
    if (!paddle1Texture.initialize(graphics,PADDLE1_IMAGE))
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing paddle texture"));
	
	// paddle2 texture
    if (!paddle2Texture.initialize(graphics,PADDLE2_IMAGE))
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing paddle texture"));

	// ball image
	if (!ball.initialize(this,ballNS::WIDTH,ballNS::HEIGHT,0,&ballTexture))
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing paddle"));
				
		// Seed the rand function
		srand(time(NULL));
		ball.setX(GAME_WIDTH/2 - ballNS::WIDTH/2);
		ball.setY(GAME_HEIGHT/2 - ballNS::HEIGHT/2);
		ball.setVelocity(VECTOR2(ballNS::SPEED *(rand()%8 -4),-ballNS::SPEED*(rand()%8 -4))); // VECTOR2(X, Y)

    // paddle1 image
	if (!paddle1.initialize(this,paddleNS::WIDTH,paddleNS::HEIGHT,0,&paddle1Texture))
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing paddle"));
		paddle1.setDegrees(180);

	// paddle2 image
	if (!paddle2.initialize(this,paddleNS::WIDTH,paddleNS::HEIGHT,0,&paddle2Texture))
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing paddle"));
		paddle2.setDegrees(0);

    // background texture
    if (!backgroundTexture.initialize(graphics,BACKGROUND_IMAGE))
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing background texture"));

    // background image
    if (!background.initialize(graphics,0,0,0,&backgroundTexture))
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing background"));
    
	// initialize DirectX font
    // 18 pixel high Arial
    if(dxFont->initialize(graphics, 18, true, false, "Arial") == false)
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing DirectX font"));

	audio->playCue(START);
	ball.update(frameTime);
	paddle1.update(frameTime);
	paddle2.update(frameTime);

    return;
}

//=============================================================================
// Update all game items
//=============================================================================
void Pong::update()
{
		ball.update(frameTime);
	
		paddle1.update(frameTime);

		paddle2.update(frameTime);

		paddle1.setDegrees(paddle1.getDegrees() + paddle1.getVelocityX() );
		paddle2.setDegrees(paddle2.getDegrees() + paddle2.getVelocityY() );	
		paddle1.setVelocityX(paddle1.getVelocityX() + paddle1.getVelocityX() * -.004);
		paddle2.setVelocityY(paddle2.getVelocityY() + paddle2.getVelocityY() * -.004);
}

//=============================================================================
// Artificial Intelligence
//=============================================================================
void Pong::ai()
{}

//=============================================================================
// Handle collisions
//=============================================================================
void Pong::collisions()
{

	
if (28> (abs(((int)(paddle1.getDegrees() - paddle2.getDegrees()+14))% 360)))
	{
	 paddle1.setVelocityX(paddle1.getVelocityX() + paddle2.getVelocityY() * -.05);
	 paddle2.setVelocityY(paddle2.getVelocityY() + paddle1.getVelocityX() * -.05); 
	 paddle1.setVelocityX(paddle1.getVelocityX() * -1.5);
	 paddle2.setVelocityY(paddle2.getVelocityY() * -1.5);
	
	}


    if (ball.getX() > GAME_WIDTH - 20 || ball.getX() < 0 || 
		ball.getY() > GAME_HEIGHT - 20 || ball.getY() < 0)
	{
		if ( winner == 0)
		{score1 += 1;}
		if ( winner == 1)
		{score2 += 1;}

		winner = -1;

		do
		ball.setVelocity(VECTOR2(ballNS::SPEED *(rand()%8 -4),-ballNS::SPEED*(rand()%8 -4))); // VECTOR2(X, Y)

		while (ball.getVelocityX() == 0 && ball.getVelocityY() == 0);

		ball.setX(GAME_WIDTH/2 - ballNS::WIDTH/2);
		ball.setY(GAME_HEIGHT/2 - ballNS::HEIGHT/2);
	}


		VECTOR2 cv;
    if(ball.collidesWith(paddle1, cv) && winner != 0)
	  {
		  ball.bounce(cv, paddle1);
		  winner = 0;
	  }
	if(ball.collidesWith(paddle2, cv) && winner != 1)
	   {
		  ball.bounce(cv, paddle2);
		  winner = 1;
	  }

	if(ball.collidesWith(paddle1, cv) && winner == 0)
	  {
		 score1 -= 2;
		 winner = 1;
	  }
	if(ball.collidesWith(paddle2, cv) && winner == 1)
	   {
		 score2 -= 2;
		  winner = 0;
	  }

		

} 
//=============================================================================
// Render game items
//=============================================================================
void Pong::render()
{
    graphics->spriteBegin();                // begin drawing sprites
    background.draw();
	paddle1.draw();
	paddle2.draw();
	ball.draw();
    dxFont->setFontColor(graphicsNS::ORANGE);
	dxFont->print(std::to_string(score1),GAME_WIDTH/8,GAME_HEIGHT/32);
	dxFont->print(std::to_string(score2),GAME_WIDTH/8*7,GAME_HEIGHT/32);
    graphics->spriteEnd();                  // end drawing sprites
}

//=============================================================================
// The graphics device was lost.
// Release all reserved video memory so graphics device may be reset.
//=============================================================================
void Pong::releaseAll()
{
	backgroundTexture.onLostDevice();
    dxFont->onLostDevice();
    Game::releaseAll();
    return;
}

//=============================================================================
// The grahics device has been reset.
// Recreate all surfaces.
//=============================================================================
void Pong::resetAll()
{
    backgroundTexture.onResetDevice();
    dxFont->onResetDevice();
    Game::resetAll();
    return;
}
